import 'package:api/models/requests/authentication/user_login_request.dart';
import 'package:api/models/requests/authentication/user_register_request.dart';
import 'package:api/models/responses/authentication/interfaces/authentication_response.dart';
import 'package:api/services/interfaces/authentication_service.dart';
import 'package:api/authentication/http/http_request_secret_validator.dart';

abstract class AuthenticationApiEndpoint<TRequest, TResponse, TApplicationSecret, TUserSecret> {
  HttpRequestSecretValidator<TRequest, TApplicationSecret> get applicationValidator;
  AuthenticationService<TUserSecret> get authenticationService;

  Future<UserRegisterRequest> parseRegisterRequest(TRequest request);
  Future<TResponse> createRegisterResponse(AuthenticationResponse<TUserSecret> response);
  Future<UserLoginRequest> parseLoginRequest(TRequest request);
  Future<TResponse> createLoginResponse(AuthenticationResponse<TUserSecret> response);

  Future<TResponse> register(TRequest request) async {
    await applicationValidator.validate(request);

    var registerRequest = await parseRegisterRequest(request);
    var authenticationResponse = await authenticationService.register(registerRequest);
    return createRegisterResponse(authenticationResponse);
  }

  Future<TResponse> login(TRequest request) async {
    await applicationValidator.validate(request);

    var loginRequest = await parseLoginRequest(request);
    var authenticationResponse = await authenticationService.login(loginRequest);
    return createLoginResponse(authenticationResponse);
  }
}
