import 'package:api/authentication/http/http_request_secret_validator.dart';
import 'package:api/models/domain/point.dart';
import 'package:api/models/requests/point/point_create_request.dart';
import 'package:api/models/requests/point/point_get_request.dart';
import 'package:api/services/interfaces/point_service.dart';

abstract class PointApiEndpoint<TRequest, TResponse, TApplicationSecret, TUserSecret, TPointId> {
  HttpRequestSecretValidator<TRequest, TApplicationSecret> get applicationValidator;
  HttpRequestSecretValidator<TRequest, TUserSecret> get userValidator;
  PointService<TPointId> get pointService;

  Future<PointCreateRequest> parseCreateRequest(TRequest request);
  Future<PointGetRequest<TPointId>> parseGetRequest(TRequest request);

  Future<TResponse> createPointResponse(Point<TPointId> response);
  Future<TResponse> createPointGetResponse(Stream<Point<TPointId>> response);

  Future<TResponse> create(TRequest request) async {
    await applicationValidator.validate(request);
    await userValidator.validate(request);

    var createRequest = await parseCreateRequest(request);
    var createResponse = await pointService.create(createRequest);
    return createPointResponse(createResponse);
  }

  Future<TResponse> get(TRequest request) async {
    await applicationValidator.validate(request);
    await userValidator.validate(request);

    var getRequest = await parseGetRequest(request);
    var getResponse = pointService.get(getRequest);
    return createPointGetResponse(getResponse);
  }
}
