import 'package:api/models/requests/user/user_update_request.dart';
import 'package:api/models/responses/user/user_update_response.dart';
import 'package:api/services/interfaces/user_service.dart';
import 'package:api/authentication/http/http_request_secret_validator.dart';

abstract class UserApiEndpoint<TRequest, TResponse, TUserId, TUserSecret, TApplicationSecret> {
  HttpRequestSecretValidator<TRequest, TApplicationSecret> get applicationValidator;
  HttpRequestSecretValidator<TRequest, TUserSecret> get userValidator;
  UserService<TUserSecret> get userService;

  Future<UserUpdateRequest<TUserId>> parseUpdateRequest(TRequest request);
  Future<TResponse> createUpdateResponse(UserUpdateResponse<TUserSecret> response);

  Future<TResponse> update(TRequest request) async {
    await applicationValidator.validate(request);
    await userValidator.validate(request);

    var parsedRequest = await parseUpdateRequest(request);
    var updateResponse = await userService.update(parsedRequest);
    return createUpdateResponse(updateResponse);
  }
}
