import 'package:api/api_endpoints/abstract/authentication_api_endpoint.dart';
import 'package:api/converters/requests/authentication/interfaces/user_login_request_converter.dart';
import 'package:api/converters/requests/authentication/interfaces/user_register_request_converter.dart';
import 'package:api/converters/responses/authentication/interfaces/authentication_response_converter.dart';
import 'package:api/models/requests/authentication/user_login_request.dart';
import 'package:api/models/requests/authentication/user_register_request.dart';
import 'package:api/models/responses/authentication/interfaces/authentication_response.dart';
import 'package:api/services/interfaces/authentication_service.dart';
import 'package:api/authentication/http/http_request_secret_validator.dart';
import 'package:api/utils/shelf/shelf_utils.dart';
import 'package:shelf/shelf.dart';
import 'package:shelf_router/shelf_router.dart';

part 'shelf_authentication_api_endpoint.g.dart';

class ShelfAuthenticationApiEndpoint<TApplicationSecret, TUserSecret> 
  extends AuthenticationApiEndpoint<Request, Response, TApplicationSecret, TUserSecret> 
{
  final UserRegisterRequestConverter<Request> registerRequestConverter;
  final UserLoginRequestConverter<Request> loginRequestConverter;
  final AuthenticationResponseConverter<Response, TUserSecret> authenticationResponseConverter;

  @override
  final HttpRequestSecretValidator<Request, TApplicationSecret> applicationValidator;

  @override
  final AuthenticationService<TUserSecret> authenticationService;

  ShelfAuthenticationApiEndpoint(
    this.authenticationService,
    this.applicationValidator,
    this.registerRequestConverter,
    this.loginRequestConverter,
    this.authenticationResponseConverter,
  );

  Router get router => _$ShelfAuthenticationApiEndpointRouter(this);

  @override
  Future<Response> createLoginResponse(AuthenticationResponse<TUserSecret> response) {
    return Future.value(authenticationResponseConverter(response));
  }

  @override
  Future<Response> createRegisterResponse(AuthenticationResponse<TUserSecret> response) {
    return Future.value(authenticationResponseConverter(response));
  }

  @override
  Future<UserLoginRequest> parseLoginRequest(Request request) {
    return loginRequestConverter(request);
  }

  @override
  Future<UserRegisterRequest> parseRegisterRequest(Request request) {
    return registerRequestConverter(request);
  }

  @override
  @Route.post('/register/')
  Future<Response> register(Request request) {
    return ShelfUtils.executeErrorHandled(super.register, request);
  }

  @override
  @Route.post('/login/')
  Future<Response> login(Request request) {
    return ShelfUtils.executeErrorHandled(super.login, request);
  }
}
