import 'package:api/api_endpoints/abstract/point_api_endpoint.dart';
import 'package:api/authentication/http/http_request_secret_validator.dart';
import 'package:api/converters/requests/point/interfaces/point_create_request_converter.dart';
import 'package:api/converters/requests/point/interfaces/point_get_request_converter.dart';
import 'package:api/converters/responses/point/interfaces/point_get_response_converter.dart';
import 'package:api/converters/responses/point/interfaces/point_response_converter.dart';
import 'package:api/models/domain/point.dart';
import 'package:api/models/requests/point/point_create_request.dart';
import 'package:api/models/requests/point/point_get_request.dart';
import 'package:api/services/interfaces/point_service.dart';
import 'package:api/utils/shelf/shelf_utils.dart';
import 'package:shelf/shelf.dart';
import 'package:shelf_router/shelf_router.dart';

part 'shelf_point_api_endpoint.g.dart';

class ShelfPointApiEndpoint<TApplicationSecret, TUserSecret, TPointId> 
  extends PointApiEndpoint<Request, Response, TApplicationSecret, TUserSecret, TPointId>
{
  static const String route = '/';

  final PointCreateRequestConverter<Request> pointCreateRequestConverter;
  final PointGetRequestConverter<Request, TPointId> pointGetRequestConverter;
  final PointResponseConverter<Response, TPointId> pointResponseConverter;
  final PointGetResponseConverter<Response, TPointId> pointGetResponseConverter;

  @override
  final HttpRequestSecretValidator<Request, TApplicationSecret> applicationValidator;

  @override
  final HttpRequestSecretValidator<Request, TUserSecret> userValidator;

  @override
  final PointService<TPointId> pointService;

  ShelfPointApiEndpoint(
    this.pointService,
    this.applicationValidator,
    this.userValidator,
    this.pointCreateRequestConverter,
    this.pointGetRequestConverter,
    this.pointResponseConverter,
    this.pointGetResponseConverter,
  );

  Router get router => _$ShelfPointApiEndpointRouter(this);

  @override
  Future<Response> createPointResponse(Point<TPointId> response) {
    return Future.value(pointResponseConverter(response));
  }
  
  @override
  Future<Response> createPointGetResponse(Stream<Point<TPointId>> response) async {
    var points = await response.toList();
    return pointGetResponseConverter(points);
  }

  @override
  Future<PointCreateRequest> parseCreateRequest(Request request) {
    return pointCreateRequestConverter(request);
  }

  @override
  Future<PointGetRequest<TPointId>> parseGetRequest(Request request) {
    return pointGetRequestConverter(request);
  }

  @override
  @Route.post(route)
  Future<Response> create(Request request) {
    return ShelfUtils.executeErrorHandled(super.create, request);
  }

  @override
  @Route.get(route)
  Future<Response> get(Request request) {
    return ShelfUtils.executeErrorHandled(super.get, request);
  }
}
