import 'package:api/api_endpoints/abstract/user_api_endpoint.dart';
import 'package:api/authentication/authentication_providers/interfaces/authentication_item_retriever.dart';
import 'package:api/authentication/http/interfaces/http_request_secret_retriever.dart';
import 'package:api/converters/requests/user/interfaces/user_update_request_converter.dart';
import 'package:api/converters/responses/user/interfaces/user_update_response_converter.dart';
import 'package:api/models/requests/user/user_update_request.dart';
import 'package:api/models/responses/user/user_update_response.dart';
import 'package:api/services/interfaces/user_service.dart';
import 'package:api/authentication/http/http_request_secret_validator.dart';
import 'package:api/utils/shelf/shelf_utils.dart';
import 'package:shelf/shelf.dart';
import 'package:shelf_router/shelf_router.dart';

part 'shelf_user_api_endpoint.g.dart';

class ShelfUserApiEndpoint<TUserId, TUserSecret, TApplicationSecret> 
  extends UserApiEndpoint<Request, Response, TUserId, TUserSecret, TApplicationSecret>
{
  final UserUpdateRequestConverter<Request, TUserId> updateRequestConverter;
  final UserUpdateResponseConverter<Response, TUserSecret> updateResponseConverter;
  final HttpRequestSecretRetriever<Request, TUserSecret> userSecretRetriever;
  final AuthenticationItemRetriever<TUserId, TUserSecret> userIdRetriever;

  @override
  final UserService<TUserSecret> userService;

  @override
  final HttpRequestSecretValidator<Request, TApplicationSecret> applicationValidator;

  @override
  HttpRequestSecretValidator<Request, TUserSecret> userValidator;

  Router get router => _$ShelfUserApiEndpointRouter(this);

  ShelfUserApiEndpoint(
    this.userService,
    this.applicationValidator,
    this.userValidator,
    this.userSecretRetriever,
    this.userIdRetriever,
    this.updateRequestConverter,
    this.updateResponseConverter,
  );

  @override
  Future<UserUpdateRequest<TUserId>> parseUpdateRequest(Request request) async {
    var userSecret = await userSecretRetriever.retrieve(request);
    var userId = await userIdRetriever.retrieve(userSecret);
    return updateRequestConverter(request, userId);
  }
  
  @override
  Future<Response> createUpdateResponse(UserUpdateResponse<TUserSecret> response) {
    return Future.value(updateResponseConverter(response));
  }

  @override
  @Route.put('/')
  Future<Response> update(Request request) {
    return ShelfUtils.executeErrorHandled(super.update, request);
  }
}
