import 'package:api/authentication/authentication_providers/interfaces/authentication_validator.dart';
import 'package:api/authentication/authentication_providers/interfaces/authenticator.dart';
import 'package:api/exceptions/not_authenticated_exception.dart';
import 'package:dart_jsonwebtoken/dart_jsonwebtoken.dart';

abstract class JwtAuthenticationProvider<TAuthenticationItem> implements 
  Authenticator<TAuthenticationItem, String>,
  AuthenticationValidator<String>
{
  SecretKey get secretKey;
  AuthenticationExceptionType get authenticationExceptionType;

  @override
  Future<void> validate(String token) {
    try {
      JWT.verify(token, secretKey);
      return Future.value();
    } 
    on JWTInvalidException catch (_) {}
    on JWTExpiredException catch (_) {}
    on JWTNotActiveException catch (_) {}
    throw NotAuthenticatedException(authenticationExceptionType);
  }

  String createToken(Map<String, dynamic> payload, Duration expiresIn, {Duration? notBefore, bool issuedAt = true}) {
    var jwt = JWT(payload);
    return jwt.sign(
      secretKey,
      expiresIn: expiresIn,
      notBefore: notBefore,
      noIssueAt: !issuedAt,
      algorithm: JWTAlgorithm.HS256,
    );
  }
}
