import 'package:api/authentication/authentication_providers/jwt/abstract/jwt_authentication_provider.dart';
import 'package:api/exceptions/not_authenticated_exception.dart';
import 'package:api/models/domain/user.dart';
import 'package:dart_jsonwebtoken/dart_jsonwebtoken.dart';

class JwtUserAuthenticationProvider<TUserId> extends JwtAuthenticationProvider<User<TUserId>> {
  static const String tokenUserIdFieldName = 'userId';
  final Duration tokenExpiresIn = Duration(minutes: 5);

  @override
  final SecretKey secretKey;
  
  @override
  final AuthenticationExceptionType authenticationExceptionType = AuthenticationExceptionType.userSecret;

  JwtUserAuthenticationProvider(String secretKey) :
    secretKey = SecretKey(secretKey);

  @override
  Future<String> authenticate(User<TUserId> user) {
    var payload = <String, dynamic>{
      tokenUserIdFieldName: user.id,
    };

    return Future.value(createToken(payload, tokenExpiresIn));
  }
}
