import 'package:api/authentication/authentication_providers/interfaces/authentication_item_retriever.dart';
import 'package:api/authentication/authentication_providers/jwt/jwt_user_authentication_provider.dart';
import 'package:api/exceptions/authentication_format_exception.dart';
import 'package:dart_jsonwebtoken/dart_jsonwebtoken.dart';

class JwtUserAuthenticationUserIdRetriever<TUserId> implements AuthenticationItemRetriever<TUserId, String> {
  @override
  Future<TUserId> retrieve(String token) async {
    var jwt = JWT.tryDecode(token);
    if(jwt == null) {
      throw AuthenticationFormatException("Not a valid JWT token");
    }

    var payload = jwt.payload as Map<String, dynamic>;
    if(!payload.keys.contains(JwtUserAuthenticationProvider.tokenUserIdFieldName)) {
      throw AuthenticationFormatException("JWT token does not contain ${JwtUserAuthenticationProvider.tokenUserIdFieldName}");
    }

    var userId = payload[JwtUserAuthenticationProvider.tokenUserIdFieldName];

    if (userId is! TUserId) {
      throw AuthenticationFormatException("Property ${JwtUserAuthenticationProvider.tokenUserIdFieldName} of JWT token is expected to be type"
        " ${TUserId.toString()}, but is type ${userId.runtimeType.toString()}");
    }

    return Future.value(userId);
  }
}
