import 'package:api/authentication/authentication_providers/interfaces/authentication_validator.dart';
import 'package:api/authentication/http/interfaces/http_request_secret_retriever.dart';

// helper class to make http request validation easier & less overhead in api endpoints
class HttpRequestSecretValidator<TRequest, TSecret> {
  final HttpRequestSecretRetriever<TRequest, TSecret> secretRetriever;
  final AuthenticationValidator<TSecret> secretValidator;

  HttpRequestSecretValidator(
    this.secretRetriever,
    this.secretValidator,
  );

  Future<void> validate(TRequest request) async {
    var secret = await secretRetriever.retrieve(request);
    return secretValidator.validate(secret);
  }
}
