import 'dart:convert';
import 'dart:io';

import 'package:api/models/responses/user/user_update_response.dart';
import 'package:api/utils/http/parameters/authentication.dart';

class HttpGeneralizedUserUpdateResponseBuilder<TUserSecret> {
  final UserUpdateResponse<TUserSecret> response;

  HttpGeneralizedUserUpdateResponseBuilder(
    this.response,
  );

  String? buildBody() {
    if(!response.wasUpdated) {
      return null;
    }
    
    if (response.authentication == null) {
      return null;
    }

    return jsonEncode({
      secretFieldName: response.authentication!.secret,
    });
  }

  int buildStatusCode() {
    if (!response.wasUpdated) {
      return HttpStatus.notModified;
    }

    return HttpStatus.ok;
  }
}
