import 'package:angel3_orm/angel3_orm.dart';
import 'package:api/converters/angel3/point_converter.dart';
import 'package:api/databases/interfaces/point_database.dart';
import 'package:api/models/databases/angel3/models.dart';
import 'package:api/models/domain/point.dart' as dm;
import 'package:uuid/uuid.dart';

class Angel3PointDatabase implements PointDatabase<String> {
  final QueryExecutor _executor;

  Angel3PointDatabase(this._executor);
  
  @override
  Future<dm.Point<String>> create(dm.Point<String> item) async {
    var currentTime = DateTime.now().toUtc();

    var query = PointQuery();
    query.values
      ..id = item.id
      ..latitude = item.latitude.toDouble()
      ..longitude = item.longitude.toDouble()
      ..recordedAt = item.recordedAt
      ..createdAt = currentTime
      ..updatedAt = currentTime;

    // var t = CarQuery();
    // t.where!.model

    var insertedPoint = await query.insert(_executor);
    return insertedPoint.value.toDomainPoint();
  }

  @override
  Future<dm.Point<String>> delete(String id) async {
    var query = PointQuery();
    query.where!
      .id.equals(id);

    var point = await query.deleteOne(_executor);
    return point.value.toDomainPoint();
  }

  @override
  Future<String> generateId() {
    return Future.value(Uuid().v4());
  }

  @override
  Stream<dm.Point<String>> getFromQuery({String? id, double? latitude, double? longitude, DateTime? createdAt}) async* {
    var query = PointQuery();

    if (id != null) {
      query.where!.id.equals(id);
    }

    if (latitude != null) {
      query.where!.latitude.equals(latitude);
    }

    if (longitude != null) {
      query.where!.longitude.equals(longitude);
    }

    if (createdAt != null) {
      query.where!.recordedAt.equals(createdAt);
    }

    var points = await query.get(_executor);
    yield* Stream.fromIterable(points.map((p) => p.toDomainPoint()));
  }

  @override
  Future<dm.Point<String>> read(String id) async {
    var query = PointQuery();
    query.where!
      .id.equals(id);

    var point = await query.getOne(_executor);
    return point.value.toDomainPoint();
  }

  @override
  Stream<dm.Point<String>> readAll() async* {
    var query = PointQuery();
    var points = await query.get(_executor);

    yield* Stream.fromIterable(points.map((p) => p.toDomainPoint()));
  }

  @override
  Future<dm.Point<String>> update(dm.Point<String> item) async {
    var query = PointQuery();

    query.where!.id.equals(item.id);
    query.values
      .. id = item.id
      ..latitude = item.latitude.toDouble()
      ..longitude = item.longitude.toDouble()
      ..recordedAt = item.recordedAt
      ..updatedAt = DateTime.now().toUtc();

    var point = await query.updateOne(_executor);
    return point.value.toDomainPoint();
  }
}
