// All database models in one file is quite ugly, this is the explanation why it is necessary.
//
// Explanation of angel3 touch points:
// - @serialize generates a (model) class, which implements the annotated class.
//   This generated class is instantiated e.g.. when making database requests.
// - @orm generates classes to query the database.
//
// As angel3_serialize (@serialize) requires the class name to start with an underscore, which makes the class private/file-scoped.
// To allow dependencies (~foreign keys etc.), the class, which defines the relation, must have access to the @orm annotated class
// (which is private because of the @serialize annotation).
// The build_runner is failing when the generated model is referenced and not the class annotated with @orm.
// This is what i've tried but doesn't work:
// - Omitting @serialize annotation and omitting leading underscore -> generated code tries to instantiate abstract class.
// - Omitting @serialize annotation and implement Model -> relation can't be created by build_runner.
// - Omitting @serialize annotation, implement and annotate it with @serialize -> multiple classes with the same name.
// - Making @orm an instantiable class -> code generation for fields does not work, e.g. Query classes don't have fields in values.
// - Changing prefix -> No possible, underscore is hardcoded in https://github.com/dart-backend/angel/blob/master/packages/serialize/angel_serialize_generator/lib/serialize.dart#L197.
// - Prefixing the class with a custom, non-private prefix, something like $_, has it's own implications such as the generated code 
//   instantiates an abstract or non-existent class.
// 
// Currently, I see two options, either create one file for all models, or fork angel3_serialize and change this single character.
// From my point of view, it would be cleaner to fork the project, which in turn implies a huge additional expenditure in maintaining and
// continuously updating the new library which i'm not really willing to take at this point.
// Therefore, all angel3 models will be placed in this file at the moment.
//
// Feel free to experiment and maybe come up with better solution, where the classes can be placed in separate files.

import 'package:angel3_orm/angel3_orm.dart';
import 'package:angel3_serialize/angel3_serialize.dart';
import 'package:api/models/databases/angel3/abstract/angel3_base_model.dart';
import 'package:api/models/domain/point.dart';
import 'package:api/models/domain/user.dart';
import 'package:optional/optional.dart';

part 'models.g.dart';

const usersTableName = 'users';

const userUsernameColumnName = 'username';
const userPasswordHashColumnName = 'password_hash';
const userNameColumnName = 'name';
const userIsEnabledColumnName = 'is_enabled';

@serializable
@Orm(generateMigrations: false, tableName: usersTableName)
abstract class _Angel3User extends Angel3BaseModel implements User<String> {
  @override
  @Column(name: idColumnName, indexType: IndexType.primaryKey, type: ColumnType.nText)
  String get id;

  @override
  @Column(name: userUsernameColumnName, indexType: IndexType.unique)
  String get username;

  @override
  @Column(name: userPasswordHashColumnName)
  String get passwordHash;

  @override
  @Column(name: userNameColumnName)
  String? get name;

  @override
  @Column(name: userIsEnabledColumnName)
  bool get isEnabled;

  @HasMany(localKey: idColumnName, foreignKey: pointUserFkName)
  List<_Angel3Point> get points;
}

const String pointsTableName = 'points';

const String pointLatitudeColumnName = 'latitude';
const String pointLongitudeColumnName = 'longitude';
const String pointRecordedAtColumnName = 'recorded_at';
const String pointUserFkName = 'user_id';

@serializable
@Orm(generateMigrations: false, tableName: pointsTableName)
abstract class _Angel3Point extends Angel3BaseModel implements Point<String> {
  @override
  @Column(name: idColumnName, indexType: IndexType.primaryKey, type: ColumnType.nText)
  String get id;

  @override
  @Column(name: pointLatitudeColumnName, type: ColumnType.decimal)
  double get latitude;

  @override
  @Column(name: pointLongitudeColumnName, type: ColumnType.decimal)
  double get longitude;

  @override
  @Column(name: pointRecordedAtColumnName)
  DateTime get recordedAt;

  @BelongsTo(localKey: pointUserFkName, foreignKey: idColumnName)
  _Angel3User get user;
}
