import 'package:api/databases/interfaces/point_database.dart';
import 'package:api/models/domain/point.dart';
import 'package:api/models/requests/point/point_create_request.dart';
import 'package:api/models/requests/point/point_get_request.dart';
import 'package:api/services/interfaces/point_service.dart';

class DefaultPointService<TPointId> implements PointService<TPointId> {
  final PointDatabase<TPointId> pointDatabase;

  DefaultPointService(
    this.pointDatabase,
  );

  @override
  Future<Point<TPointId>> create(PointCreateRequest request) async {
    var pointId = await pointDatabase.generateId();
    var point = Point(pointId, request.latitude, request.longitude, request.recordedAt);

    return pointDatabase.create(point);
  }

  @override
  Stream<Point<TPointId>> get(PointGetRequest<TPointId> request) {
    return pointDatabase.getFromQuery(
      id: request.id,
      latitude: request.latitude,
      longitude: request.longitude,
    );
  }
}
