import 'package:api/exceptions/request_attribute_format_exception.dart';
import 'package:api/exceptions/request_attribute_wrong_type_exception.dart';
import 'package:api/exceptions/request_missing_attribute_exception.dart';

abstract class RequestsUtil {
  static T? getNullableFromRequest<T>(dynamic value, String attributeName) {
    if (value == null) {
      return null;
    }

    return _checkTypeAndReturn<T>(value, attributeName);
  }

  static T getFromRequest<T>(dynamic value, String attributeName, AttributeLocation attributeLocation) {
    if (value == null) {
      throw RequestMissingAttributeException(attributeName, attributeLocation);
    }

    return _checkTypeAndReturn(value, attributeName);
  }

  static DateTime getDateTimeFromRequest(dynamic value, String attributeName, AttributeLocation attributeLocation) {
    var dateTimeAsString = getFromRequest<String>(value, attributeName, attributeLocation);
    var dateTime = DateTime.tryParse(dateTimeAsString);

    if (dateTime == null) {
      throw RequestAttributeFormatException(attributeName, 'ISO8601 String');
    }

    return dateTime;
  }

  static T _checkTypeAndReturn<T>(dynamic value, String attributeName) {
    if(value is! T) {
      throw RequestAttributeWrongTypeException(attributeName, value.runtimeType, T);
    }

    return value;
  }
}
